import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:flutter_query/src/query.dart';
import 'package:provider/provider.dart';

AsyncSnapshot<T> useQuery<T>(String key, QueryFunction fn) {
  return use(_QueryHook(Query(key, fn)));
}

class _QueryHook<T> extends Hook<AsyncSnapshot<T>> {
  const _QueryHook(this.query);

  final Query query;

  @override
  _QueryStateHook<T> createState() => _QueryStateHook();
}

class _QueryStateHook<T> extends HookState<AsyncSnapshot<T>, _QueryHook<T>> {
  AsyncSnapshot<T> _snapshot;

  @override
  void initHook() {
    var query = hook.query;

    try {
      final cache = Provider.of<QueryCache>(context, listen: false);
      query = cache.add(hook.query);
    } catch (e) {
      print('Cache not found!!! Caching is disabled for this query');
    }
    _snapshot = AsyncSnapshot<T>.withData(ConnectionState.none, null);
    query.invalidate();
    query.watch((data) {
      if (data != null) {
        setState(() {
          _snapshot = AsyncSnapshot<T>.withData(ConnectionState.done, data);
        });
      }
    });
    super.initHook();
  }

  @override
  AsyncSnapshot<T> build(BuildContext context) {
    return _snapshot;
  }
}
