import 'dart:async';

import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:rxdart/rxdart.dart';

typedef Future QueryFunction();

class Query {
  String key;
  QueryFunction fn;
  BehaviorSubject _dataStream = BehaviorSubject();

  Query(this.key, this.fn);

  void watch(Function watcher) {
    _dataStream.stream.listen((data) => watcher(data));
  }

  void invalidate() {
    fn().then((data) => _dataStream.add(data));
  }
}

class QueryCache {
  List<Query> _queries = [];

  Query getQueryByKey(String key) {
    try {
      return _queries.where((q) => q.key == key).first;
    } catch (e) {
      return null;
    }
  }

  Query add(Query query) {
    final existingQuery = getQueryByKey(query.key);
    if (existingQuery == null) {
      _queries.add(query);
      return query;
    } else {
      return existingQuery;
    }
  }

  void invalidate(String key) {
    getQueryByKey(key).invalidate();
  }
}

class QueryCacheProvider extends StatefulWidget {
  final Widget child;
  QueryCacheProvider({this.child});

  @override
  State<StatefulWidget> createState() => QueryCacheProviderState();
}

class QueryCacheProviderState extends State<QueryCacheProvider> {
  QueryCache _queryCache;

  @override
  void initState() {
    _queryCache = QueryCache();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Provider.value(
      value: _queryCache,
      child: widget.child,
    );
  }
}
